@@ -1,8 +1,8 @@
 # -*- coding: utf-8 -*-
 #
-# Copyright 2010-2019 Univention GmbH
+# Copyright 2010-2021 Univention GmbH
 #
-# http://www.univention.de/
+# https://www.univention.de/
 #
 # All rights reserved.
 #
@@ -25,20 +25,35 @@
 # You should have received a copy of the GNU Affero General Public
 # License with the Debian GNU/Linux or Univention distribution in file
 # /usr/share/common-licenses/AGPL-3; if not, see
-# <http://www.gnu.org/licenses/>.
+# <https://www.gnu.org/licenses/>.
 
 import cracklib
 import os
+import re
 import univention.uldap
+import univention.debug as ud
 import univention.config_registry as ucr
+from ldap.filter import filter_format
+
+try:
+	from samba import check_password_quality as samba_check_password_quality
+except ImportError:
+	def samba_check_password_quality(*args, **kwargs):
+		ud.debug(ud.LDAP, ud.ERROR, 'samba_check_password_quality() is not available in Python 2. Not checking password quality.')
+		return True  # not available, use Python 3
 
 
-class Check:
+class CheckFailed(Exception):
+	pass
+
+
+class Check(object):
 
 	def __init__(self, lo, username=None):
 		self.ConfigRegistry = ucr.ConfigRegistry()
 		self.ConfigRegistry.load()
 
+		self.username = username
 		self.enableQualityCheck = False
 		self.checkHistory = False
 		self.min_length = -1
@@ -49,8 +64,8 @@ class Check:
 
 		self._systemPolicy()
 
-		if username:
-			self._userPolicy(username)
+		if self.username:
+			self._userPolicy(self.username)
 
 	def _getConnection(self):
 		if os.path.exists('/etc/ldap.secret'):
@@ -82,52 +97,75 @@ class Check:
 		if self.ConfigRegistry.get('password/quality/diff_ok', None):
 			cracklib.DIFF_OK = int(self.ConfigRegistry.get('password/quality/diff_ok'))
 
-	def _userPolicy(self, username):
-		self.username = username
+		# optionally activate Microsoft standard criteria
+		self.mspolicy = self.ConfigRegistry.get('password/quality/mspolicy', None)
+		# normalize True values
+		self.mspolicy = self.ConfigRegistry.is_true(value=self.mspolicy) or self.mspolicy
 
+	def _userPolicy(self, username):
 		# username or kerberos principal
 		try:
 			if '@' in self.username:
-				dn = self.lo.searchDn('krb5PrincipalName=%s' % username)[0]
+				dn = self.lo.searchDn(filter_format('krb5PrincipalName=%s', [username]))[0]
 			else:
-				dn = self.lo.searchDn('(&(uid=%s)(|(&(objectClass=posixAccount)(objectClass=shadowAccount))(objectClass=sambaSamAccount)(&(objectClass=person)(objectClass=organizationalPerson)(objectClass=inetOrgPerson))))' % username)[0]
+				dn = self.lo.searchDn(filter_format('(&(uid=%s)(|(&(objectClass=posixAccount)(objectClass=shadowAccount))(objectClass=sambaSamAccount)(&(objectClass=person)(objectClass=organizationalPerson)(objectClass=inetOrgPerson))))', [username]))[0]
 		except IndexError:
-			raise ValueError('User was not found.')
+			raise CheckFailed('User was not found.')
 
 		policy_result = self.lo.getPolicies(dn)
 		if policy_result.get('univentionPolicyPWHistory'):
 			self.min_length = int(policy_result['univentionPolicyPWHistory']['univentionPWLength']['value'][0])
 			self.history_length = int(policy_result['univentionPolicyPWHistory']['univentionPWHistoryLen']['value'][0])
 			if policy_result['univentionPolicyPWHistory'].get('univentionPWQualityCheck'):
-				univentionPasswordQualityCheck = policy_result['univentionPolicyPWHistory']['univentionPWQualityCheck']['value'][0]
-				if univentionPasswordQualityCheck.lower() in ['yes', 'true', '1', 'on']:
-					self.enableQualityCheck = True
+				univentionPasswordQualityCheck = policy_result['univentionPolicyPWHistory']['univentionPWQualityCheck']['value'][0].decode('ASCII', 'replace')
+				self.enableQualityCheck = self.ConfigRegistry.is_true(value=univentionPasswordQualityCheck)
 		self.pwhistory = self.lo.search(base=dn, attr=['pwhistory'])[0][1].get('pwhistory')
 
-	def check(self, password):
+	def check(self, password, username=None, displayname=None):
 		if self.min_length > 0:
 			if len(password) < self.min_length:
-				raise ValueError('Password is too short')
+				raise CheckFailed('Password is too short')
 		else:
 			cracklib.MIN_LENGTH = 4  # this seems to be the lowest valid value
 
+		# Workaround for users/user, which instanciates Check() without a username:
+		# We need the username here, but if we would pass it to Check() then
+		# _userPolicy would be run, changing the behavior in users/user.
+		if not username:
+			username = self.username
+
 		# Todo: check history
 
 		if self.enableQualityCheck:
+			if self.mspolicy in (True, 'sufficient'):
+				# See https://docs.microsoft.com/de-de/windows/security/threat-protection/security-policy-settings/password-must-meet-complexity-requirements
+				if not samba_check_password_quality(password):
+					raise CheckFailed('Password does not meet the password complexity requirements.')
+				if username and len(username) > 3 and username.lower() in password.lower():
+					raise CheckFailed('Password contains user account name.')
+				if displayname:
+					for namepart in re.split('[-,._# \t]+', displayname):
+						if len(namepart) > 3 and namepart.lower() in password.lower():
+							raise CheckFailed('Password contains parts of the full user name.')
+			if self.mspolicy == 'sufficient':
+				return True  # skip all other checks
 			for c in self.forbidden_chars:
 				if c in password:
-					raise ValueError('Password contains forbidden characters')
+					raise CheckFailed('Password contains forbidden characters')
 			if self.required_chars:
 				for c in self.required_chars:
 					if c in password:
 						break
 				else:
-					raise ValueError('Password does not contain one of required characters: "%s"' % self.required_chars)
+					raise CheckFailed('Password does not contain one of required characters: "%s"' % self.required_chars)
 
 			cracklib.MIN_LENGTH = self.min_length
 
-			if cracklib.VeryFascistCheck(password) == password:
-				return True
+			try:
+				if cracklib.VeryFascistCheck(password) == password:
+					return True
+			except ValueError as exc:
+				raise CheckFailed(str(exc))
 
 
 # def test_case1():
