@@ -44,7 +44,7 @@ from ldapurl import isLDAPUrl
 import univention.debug
 from univention.config_registry import ConfigRegistry
 try:
-	from typing import Any, Dict, List, Optional, Set, Tuple, Union  # noqa
+	from typing import Any, Callable, Dict, List, Optional, Set, Tuple, Union  # noqa: F401
 except ImportError:
 	pass
 
@@ -476,7 +476,7 @@ class access(object):
 
 	@_fix_reconnect_handling
 	def search(self, filter='(objectClass=*)', base='', scope='sub', attr=[], unique=False, required=False, timeout=-1, sizelimit=0, serverctrls=None, response=None):
-		# type: (str, str, str, List[str], bool, bool, int, int, Optional[List[ldap.controls.LDAPControl]], Optional[Dict]) -> List[Tuple[str, Dict[str, List[str]]]]
+		# type: (str, str, str, List[str], bool, bool, int, int, Optional[List[ldap.controls.LDAPControl]], Optional[Dict[str, ldap.controls.LDAPControl]]) -> List[Tuple[str, Dict[str, List[str]]]]
 		"""
 		Perform LDAP search and return values.
 
@@ -522,7 +522,7 @@ class access(object):
 		return res
 
 	def searchDn(self, filter='(objectClass=*)', base='', scope='sub', unique=False, required=False, timeout=-1, sizelimit=0, serverctrls=None, response=None):
-		# type: (str, str, str, bool, bool, int, int, Optional[List[ldap.controls.LDAPControl]]) -> List[str]
+		# type: (str, str, str, bool, bool, int, int, Optional[List[ldap.controls.LDAPControl]], Optional[Dict[str, ldap.controls.LDAPControl]]) -> List[str]
 		"""
 		Perform LDAP search and return distinguished names only.
 
@@ -576,7 +576,7 @@ class access(object):
 
 		object_classes = set(oc.lower() for oc in oattrs.get('objectClass', []))
 
-		merged = {}
+		merged = {}  # type: Dict[str, Dict[str, Any]]
 		if dn:
 			obj_dn = dn
 			while True:
@@ -700,8 +700,8 @@ class access(object):
 			response['ctrls'] = resp_ctrls
 
 	@_fix_reconnect_handling
-	def modify(self, dn, changes, serverctrls=None, response=None):
-		# type: (str, List[Tuple[str, Any, Any]], Optional[List[ldap.controls.LDAPControl]], Optional[dict]) -> str
+	def modify(self, dn, changes, serverctrls=None, response=None, rename_callback=None):
+		# type: (str, List[Tuple[str, Any, Any]], Optional[List[ldap.controls.LDAPControl]], Optional[dict], Optional[Callable]) -> str
 		"""
 		Modify LDAP entry DN with attributes in changes=(attribute-name, old-values, new-values).
 
@@ -744,6 +744,8 @@ class access(object):
 		# check if we need to rename the object
 		new_dn, new_rdn = self.__get_new_dn(dn, ml)
 		if not self.compare_dn(dn, new_dn):
+			if rename_callback:
+				rename_callback(dn, new_dn, ml)
 			univention.debug.debug(univention.debug.LDAP, univention.debug.WARN, 'rename %s' % (new_rdn,))
 			self.rename_ext_s(dn, new_rdn, serverctrls=serverctrls, response=response)
 			dn = new_dn
